#include <oxstd.oxh>
#include <oxdraw.oxh>
#include <oxprob.oxh>
#include <oxfloat.oxh>
#import <maximize>

#include "Misc.ox"
#include "Base.ox"
#include "StandardizedKernelEstimator2.ox"
#include "Components.ox"

/*
	SEE PROCESS.PY
*/

main () {
	decl T = 2000;
	decl replications = 1000;
	decl K = 1000; // how much data to use for estimation

	decl mu = 0, omega = 2, alpha = 0.3, beta = 0.9;

	// six columns: mu, omega, alpha, beta, RMSE in, RMSE out
	decl MLE_results = <>;
	decl QMLE_results = <>;
	decl t_results = <>;
	decl SP_results = <>;

	decl succes = 0;
	
	while(succes < replications) {
		decl data, trueft;
		[data, trueft] = %DGP%; 
		decl truepar = mu~omega~alpha~beta;
		decl inittpar = truepar ~ <5>;

		decl par1 = truepar, code1,	ft1, epst1,
			par2 = truepar, code2, ft2, epst2,
			par3 = inittpar, code3,	ft3, epst3,
			par4 = truepar, code4, ft4, epst4;

		code1 = %MLE%(data, K, &par1, &ft1, &epst1);		
		code2 = GAS_QMLE(data, K, &par2, &ft2, &epst2);
		code3 = GAS_t(data, K, &par3, &ft3, &epst3);
		code4 = GAS_SP1(data, K, &par4, &ft4, &epst4);

		if(code1+code2+code3+code4 == 0) {
			MLE_results |= (par1 - truepar) ~ sqrt(meanr(sqr(exp(ft1[:K-1]/2) - exp(trueft[:K-1]/2)))) ~ sqrt(meanr(sqr(exp(ft1[K:]/2) - exp(trueft[K:]/2))));
			QMLE_results |= (par2 - truepar) ~ sqrt(meanr(sqr(exp(ft2[:K-1]/2) - exp(trueft[:K-1]/2)))) ~ sqrt(meanr(sqr(exp(ft2[K:]/2) - exp(trueft[K:]/2))));
			t_results |= (par3[0:3] - truepar) ~ sqrt(meanr(sqr(exp(ft3[:K-1]/2) - exp(trueft[:K-1]/2)))) ~ sqrt(meanr(sqr(exp(ft3[K:]/2) - exp(trueft[K:]/2))));
			SP_results |= (par4 - truepar) ~ sqrt(meanr(sqr(exp(ft4[:K-1]/2) - exp(trueft[:K-1]/2)))) ~ sqrt(meanr(sqr(exp(ft4[K:]/2) - exp(trueft[K:]/2))));
			succes++;
		}
		
	}

	// combine results
	decl finalresults = zeros(4, 6);
	
	finalresults[0][0:3] = sqrt(meanc(sqr(MLE_results[][0:3])));
	finalresults[1][0:3] = sqrt(meanc(sqr(QMLE_results[][0:3])));
	finalresults[2][0:3] = sqrt(meanc(sqr(t_results[][0:3])));
	finalresults[3][0:3] = sqrt(meanc(sqr(SP_results[][0:3])));
	finalresults[0][4:5] = quantilec(MLE_results[][4:5], 0.5);
	finalresults[1][4:5] = quantilec(QMLE_results[][4:5], 0.5);
	finalresults[2][4:5] = quantilec(t_results[][4:5], 0.5);
	finalresults[3][4:5] = quantilec(SP_results[][4:5], 0.5);

	finalresults[][0] /= finalresults[1][0];
	finalresults[][1] /= finalresults[1][1];
	finalresults[][2] /= finalresults[1][2];
	finalresults[][3] /= finalresults[1][3];
	finalresults[][5] /= finalresults[1][4]; // normalize this one first
	finalresults[][4] /= finalresults[1][4];

	savemat("Table1_%NAME%_runs.xlsx", MLE_results ~ QMLE_results ~ t_results ~ SP_results);

	savemat("Table1_%NAME%.xlsx", finalresults);
}