#include <oxstd.oxh>
#include <oxdraw.oxh>
#include <oxprob.oxh>
#include <oxfloat.oxh>
#import <maximize>

#include "Misc.ox"
#include "Base.ox"
#include "StandardizedKernelEstimator2.ox"
#include "Components.ox"

/*
	SEE PROCESS.PY
*/

/*
	Return the p-value for H0: P(X < Y) = 0.5, H1: P(X < Y) > 0.5

	X,Y: equal size column vectors
*/
SignTest (const x, const y) {
	decl n = rows(x);
	decl count = sumc(x .< y);
	return 1 - probbinomial(count, n, 0.5);
}

main () {
	decl T = 2000;
	decl replications = 100;
	decl K = 1000; // how much data to use for estimation

	// four columns: GAS RMSE in, GAS RMSE out, GARCH RMSE in, GARCH RMSE out
	decl QMLE_results = <>;
	decl t_results = <>;
	decl SP_results = <>;
	
	decl succes = 0;
	
	while(succes < replications) {
		decl truevol = Vol_Stoch(T);
		decl data = %RAN% .* truevol;
		decl initpar = <0, 0.5, 0.3, 0.9>;
		decl inittpar = initpar ~ <5>;
		decl initgarchpar = <0, 1.5, 0.05, 0.9>;
		decl inittgarchpar = initgarchpar ~ <5>;

		decl par1 = initpar, code1,	ft1, epst1,
			par2 = initgarchpar, code2, volsq2, epst2,
			par3 = inittpar, code3,	ft3, epst3,
			par4 = inittgarchpar, code4, volsq4, epst4,
			par5 = initpar, code5, ft5, epst5,
			par6 = initgarchpar, code6, volsq6, epst6;

		code1 = GAS_QMLE(data, K, &par1, &ft1, &epst1);		
		code2 = GARCH_QMLE(data, K, &par2, &volsq2, &epst2);
		code3 = GAS_t(data, K, &par3, &ft3, &epst3);
		code4 = GARCH_t(data, K, &par4, &volsq4, &epst4);
		if(code1+code2+code3+code4 != 0) continue;
		code5 = GAS_SP1(data, K, &par5, &ft5, &epst5);
		code6 = GARCH_SP1(data, K, &par6, &volsq6, &epst6);

		if(code1+code2+code3+code4+code5+code6 == 0) {
			QMLE_results |= sqrt(meanr(sqr(exp(ft1[:K-1]/2) - truevol[:K-1]))) ~ sqrt(meanr(sqr(exp(ft1[K:]/2) - truevol[K:])))
				~ sqrt(meanr(sqr(sqrt(volsq2[:K-1]) - truevol[:K-1]))) ~ sqrt(meanr(sqr(sqrt(volsq2[K:]) - truevol[K:])));
			t_results |= sqrt(meanr(sqr(exp(ft3[:K-1]/2) - truevol[:K-1]))) ~ sqrt(meanr(sqr(exp(ft3[K:]/2) - truevol[K:])))
				~ sqrt(meanr(sqr(sqrt(volsq4[:K-1]) - truevol[:K-1]))) ~ sqrt(meanr(sqr(sqrt(volsq4[K:]) - truevol[K:])));
			SP_results |= sqrt(meanr(sqr(exp(ft5[:K-1]/2) - truevol[:K-1]))) ~ sqrt(meanr(sqr(exp(ft5[K:]/2) - truevol[K:])))
				~ sqrt(meanr(sqr(sqrt(volsq6[:K-1]) - truevol[:K-1]))) ~ sqrt(meanr(sqr(sqrt(volsq6[K:]) - truevol[K:])));
			succes++;
		}
	}

	// combine results
	decl finalresults = zeros(3, 6);
	
	finalresults[0][0:3] = quantilec(QMLE_results[][0:3], 0.5);
	finalresults[1][0:3] = quantilec(t_results[][0:3], 0.5);
	finalresults[2][0:3] = quantilec(SP_results[][0:3], 0.5);

	finalresults ./= finalresults[0][0];

	finalresults[0][4] = SignTest(QMLE_results[][0], QMLE_results[][2]);
	finalresults[0][5] = SignTest(QMLE_results[][1], QMLE_results[][3]);
	finalresults[1][4] = SignTest(t_results[][0], t_results[][2]);
	finalresults[1][5] = SignTest(t_results[][1], t_results[][3]);
	finalresults[2][4] = SignTest(SP_results[][0], SP_results[][2]);
	finalresults[2][5] = SignTest(SP_results[][1], SP_results[][3]);

	savemat("Table3_%NAME%.xlsx", finalresults);
	savemat("Table3_%NAME%_runs.xlsx", QMLE_results ~ t_results ~ SP_results);
}